package org.freecompany.yacli;

import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.List;
import java.util.HashMap;
import java.util.ListIterator;
import java.util.Map;
import java.util.logging.Logger;

import static java.util.logging.Level.FINE;
import static java.util.logging.Level.FINER;
import static java.util.logging.Logger.getLogger;

public class CommandLine {

	public static final String SHORT_OPTION = "-{1}[^-&&\\S]+";
	public static final String LONG_OPTION = "-{2}[^-&&\\S]+";

	protected final Logger logger = getLogger( CommandLine.class.getName());
	protected final Map< Option, Field> fields = new HashMap< Option, Field>();
	protected final Options options = new Options();
	protected final Object object;
	protected final List< String> arguments = new ArrayList< String>();
	protected String usage = "";

	public CommandLine( final Object object) {
		this.object = object;
		final Usage usage = object.getClass().getAnnotation( Usage.class);
		if ( usage != null) {
			logger.log( FINE, "Setting usage ''{0}''.", usage);
			this.usage = usage.value();
		}
		for ( Field field : object.getClass().getFields()) {
			final Option option = field.getAnnotation( Option.class);
			if ( option != null) {
				final Class< ?> type = option.value().type();
				logger.log( FINE, "Adding option for field ''{0}''.", field);
				if ( !type.equals( field.getType())) throw new IllegalArgumentException( "The field type does not match the annotated value type.");
				options.add( option);
				fields.put( option, field);
			}
		}
	}

	public void parse( final String[] args) throws InvalidCommandException {
		for ( String arg : args) arguments.add( arg);
		
		try {
			for ( ListIterator< String> i = arguments.listIterator(); i.hasNext();) {
				final String current = i.next();
				Entry entry = null;
				if ( current.matches( SHORT_OPTION)) {
					final String shortName = current.substring( 1);
					entry = options.shortName( shortName);
				} else if ( current.matches( LONG_OPTION)) {
					final String longName = current.substring( 2);
					entry = options.longName( longName);
				}

				if ( entry != null) {
					i.remove();
					logger.log( FINE, "Found entry ''{0}''.", entry.option().longName());
					entry.setInvoked( true);
					entry.takeArguments( i);
					logger.log( FINE, "Option ''{0}'' took argument(s) ''{1}''.", new Object[] { entry.option().longName(), entry.getValue()});
				}	
				logger.log( FINER, "Argument list ''{0}''.", arguments);
			}
			
			options.validate();
			
			for ( Entry entry : options.getEntries()) entry.updateField( object, fields.get( entry.option()));
		} catch ( InvalidOptionException e) {
			throw new InvalidCommandException( e, options, usage);
		}
	}

	public String[] remaining() { return arguments.toArray( new String[ arguments.size()]); }
}
