package org.freecompany.yacli;

import java.lang.reflect.Field;
import java.util.ListIterator;

import static org.freecompany.yacli.Required.REQUIRED;
import static org.freecompany.yacli.Value.AT_LEAST_ONE;
import static org.freecompany.yacli.Value.EXACTLY_ONE;
import static org.freecompany.yacli.Value.NONE;
import static org.freecompany.yacli.Value.ONE_OR_NONE;

public class Entry {

	public static final String ANY_OPTION = "-{1,2}[^-&&\\S]+";
    
    protected final Option option;

	protected String[] value;
    protected boolean invoked;

    public Entry( final Option option) {
		this.option = option;
    }

	public Option option() {
		return option;
	}

    public void setInvoked( final boolean invoked) {
        this.invoked = invoked;
    }

    public boolean invoked() {
        return invoked;
    }

    public String[] getValue() {
		return value;
    }

    public void setValue( final String... value) {
		this.value = value;
    }

	public void takeArguments( final ListIterator< String> i) throws InvalidOptionException {
		String string = null;
		switch ( option.value()) {
			case EXACTLY_ONE:
				if ( !i.hasNext() || ( string = i.next()).matches( ANY_OPTION)) {
					i.previous();
					return;
				}
				i.remove();
				value = new String[] { string};
				break;
			case ONE_OR_NONE:
				if ( !i.hasNext()) return;
				if (( string = i.next()).matches( ANY_OPTION)) {
					i.previous();
					return;
				}
				i.remove();
				value = new String[] { string};
				break;
			case AT_LEAST_ONE:
				while ( i.hasNext()) {
					if (( string = i.next()).matches( ANY_OPTION)) {
						i.previous();
						return;
					}
					i.remove();
					value = new String[] { string};
				}
				break;
		}
	}

	public void validate() throws InvalidOptionException {
		if ( invoked) {
			switch ( option.value()) {
				case NONE:
					if ( value != null) throw new InvalidOptionException( "Entry '" + option.longName() + "' does not accept an argument.");
					break;
				case ONE_OR_NONE:
					if ( value != null && value.length > 1) throw new InvalidOptionException( "Entry '" + option.longName() + "' only accepts a single argument.");
					break;
				case EXACTLY_ONE:
					if ( value == null || value.length != 1) throw new InvalidOptionException( "Entry '" + option.longName() + "' requires exactly one argument.");
					break;
				case AT_LEAST_ONE:
					if ( value == null) throw new InvalidOptionException( "Entry '" + option.longName() + "' requires at least one argument.");
					break;
			}
		}
		switch ( option.required()) {
			case REQUIRED:
				if ( !invoked) throw new InvalidOptionException( "Entry '" + option.longName() + "' is required.");
				break;
		}
	}

	public void updateField( final Object object, final Field field) throws InvalidOptionException {
		if ( value == null) return;
		try {
			field.set( object, value.length == 1 ? value[ 0] : value);
		} catch ( IllegalAccessException e) {
			throw new InvalidOptionException( "Field '" + field + "' is not accessible.");
		}
	}

    public String toString() {
        StringBuffer buffer = new StringBuffer( "\t");

        if ( option.shortName() != null) buffer.append( "-").append( option.shortName());
        else buffer.append( "	");

        if ( option.longName() != null) {
            if ( option.shortName() != null) buffer.append( ",--");
            else buffer.append( " --");

            buffer.append( option.longName());
        } else {
            buffer.append( "\t");
        }

        buffer.append( "\t").append( option.description());
		if ( option.required() == REQUIRED) buffer.append( " [REQUIRED]");
		return buffer.toString();
    }
}
